@@ -16,7 +16,7 @@ from synapse.rest import admin
 from synapse.rest.client import login, room
 from synapse.server import HomeServer
 from synapse.types import JsonDict
-from synapse.util import Clock
+from synapse.util.clock import Clock
 from synapse.util.retryutils import NotRetryingDestination
 
 from tests.test_utils import event_injection
